from density_estimation import KDE, KNeighborsDensity
from xdeconv import XDGMM
from histtools import\
    scotts_bin_width, freedman_bin_width, knuth_bin_width, histogram
from bayesian_blocks import bayesian_blocks
from empirical import FunctionDistribution, EmpiricalDistribution
from gauss_mixture import GaussianMixture1D
